/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 *  epix.h -- Automatically created on Mon Apr 30 07:26:49 GMT 2007  *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <iostream>
#include <vector>
#include <list>
#include <sstream>
#include <cmath>
#include <cstdio>
#include <cstdarg>

#ifndef EPIX_H
#define EPIX_H

/* 
 * globals.h -- Top-level header file
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 10, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *  This file provides the epix class to "manage" global variables:
 *
 *  - dash_length (2 -- 200 true points; parameter same as dot_sep)
 *  - dash_fill   (0.05 -- 0.95, fraction of path filled when dashed)
 *  - dot_size    (0.5  --  256 true pt)
 *  - dot_sep
 *
 *  - gray_depth  (0=white -- 1=black)
 *  - angle_units (radians, degrees, revolutions)
 *  - path_style  (SOLID, DASHED, DOTTED)
 *
 *  - clipping
 *  - cropping
 *  - fill_paths
 */



namespace ePiX {


  enum epix_path_style  {SOLID, DASHED, DOTTED};
  //  enum epix_output_type {EEPIC}; // METAPOST, SVG...

  // for backward compatibility, not in class epix below
  extern double x_min, x_max, x_size, y_min, y_max, y_size;
  extern double tix;

  // pre-declare friends for gcc-4.1
  void dash_fill(double t=0.5);
  void dash_length(double len=12);
  void dot_sep(double len=12);
  void dot_size(double diam=3);
  void gray(double depth=0.3);

  void radians();
  void degrees();
  void revolutions();
  double angle(const double t);

  void label_angle(double t); // defined in functions.cc

  void solid();
  void dashed(double t=0);
  void dotted(double t=0);

  void clip(bool arg = true);
  void crop(bool arg = true);
  void fill(bool arg = true);
  void use_pstricks(bool arg = true);

  void font_size(std::string arg="normalsize");
  void font_face(std::string arg="default");

  void begin(); // write figure header; defined in output.cc

  void arrow_width(double w=3);
  void arrow_ratio(double r=5.5);
  void arrow_camber(double arg=0);
  void arrow_fill(double dens=0);
  // end of friend pre-declarations


  // global "style" constants
  class epix {
  public:
    static bool clipping;
    static bool cropping;
    static bool fill_paths;

    static bool using_pstricks;
    static std::string fillcolor; // no effect unless using_pstricks

    static std::string fontsize; // valid LaTeX font size
    static std::string fontface; // valid LaTeX font face

    // functions for setting style parameters
    friend void dash_fill(double t);
    friend void dash_length(double len);

    friend void dot_sep(double len);
    friend void dot_size(double diam);

    friend void gray(double depth);

    friend void radians();
    friend void degrees();
    friend void revolutions();

    // radians per angle unit
    friend double angle(const double t);

    // store label rotation angle internally in degrees
    friend void label_angle(double t);

    // set state variable
    friend void solid();
    friend void dashed(double t);
    friend void dotted(double t);

    friend void clip(bool arg);
    friend void crop(bool arg);
    friend void fill(bool arg);
    friend void use_pstricks(bool arg);

    // set font size and face
    friend void font_size(std::string arg);
    friend void font_face(std::string arg);

    friend void begin(); // defined in output.cc

    // set arrowhead parameters
    friend void arrow_width(double w);
    friend void arrow_ratio(double r);
    friend void arrow_camber(double arg);
    friend void arrow_fill(double dens);
    static double get_arrow_width();
    static double get_arrow_ratio();
    static double get_arrow_camber();
    static double get_arrow_fill();

    // for internal use
    static double get_dashfill();
    static double get_dashlength();

    static double get_gray();
    static double get_dotsize();
    static double full_turn();
    static double get_labelangle();
    static epix_path_style path_style();

  private:
    static double dashfill;    // 0.05 --   0.95
    static double separation;  // 2    -- 200 (true pt)
    static double gray_depth;  // 0    --   1
    static double dotsize;     // 0.5  --  10

    static double angle_units;
    static double labelangle;  // rotate labels this amount in degrees

    // arrowhead parameters
    static double arrowwidth; // Half-width of arrowheads in pt
    static double arrowratio; // 2*Length/width ratio of arrowheads
    static double arrowcamber;// Base indentation (fraction of width*ratio)
    static double arrowfill;  // Fill density for arrowheads

    //    static epix_output_type OUTPUT_TYPE;
    //    static std::string font;

    static epix_path_style PATH_STYLE;
    static int begin_count; // number of times begin() has been called
  }; // end of class epix


  // Compile time constants

  // Precision-related constants:
  // ePiX's idea of too small/large
  const double EPIX_EPSILON(0.0001);  // 10^-4
  const double EPIX_INFTY(100000.0);  // 10^5

  // In calculus plotting, the interval between adjacent points is too large
  // for accuracy. Divide each such interval into this many subintervals
  const int EPIX_ITERATIONS(200);

  // Output-formatting constants:
  const unsigned int EPIX_NUM_PTS(80); // # of points in ellipses, splines
  const int EPIX_FILE_WIDTH(70);  // Width of output file
  const int EPIX_PATH_LENGTH(60); // Number of points per path segment

  // Enumeration types:
  enum epix_mark_type {PATH, CIRC, SPOT, RING, DOT, DDOT, PLUS, OPLUS, 
		       TIMES, OTIMES, DIAMOND, UP, DOWN, BOX, BBOX, 
		       H_TICK, V_TICK, TEXT};

  enum epix_label_posn {none, c, r, tr, rt, t, tl, lt, l, bl, lb, b, br, rb};

  enum epix_tick_type {TICK_NULL, H_AXIS, V_AXIS};

  enum epix_field_type {SLOPE, DART, VECTOR};

  enum epix_integral_type {LEFT, RIGHT, UPPER, LOWER, TRAP, MIDPT};

  // Deprecated constant
  const int EPIX_PAIRS_PER_LINE(4);// Number of pairs per line in output file


  // <pic_unit> is one of the following valid LaTeX length units: 
  // bp, cm, in, mm, pc, pt (default), or sp.
  // <pic_size> is a double that (with pic_unit) specifies unitlength
  extern double pic_size; // e.g., 1
  extern char *pic_unit;  // e.g., "pt" 

} // end of namespace

/* 
 *  triples.h -- ePiX::P class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: Obcober 09, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The P class (ordered triple of <double>s) and operators
 *
 *    - P(), P() (default to origin), E_1, E_2, E_3 (standard basis)
 *    - Vector space operators
 *    - Other vector operators, operands (a,b,c), (x,y,z)
 *       | dot product                      ax + by + cz
 *       norm(p) = sqrt(p|p)
 *       & componentwise product            (ax, by, cz)
 *       * cross product                    (bz-cy, cx-az, ay-bx)
 *       J rotate (x1,x2,0)-plane 1/4 turn  (-b,a,0)
 *       % orthogonalization                u%v = u - ((u|v)/(v|v))*v
 */


namespace ePiX {

  class P {
  public:
    P(double arg1=0, double arg2=0, double arg3=0);

    double x1() const;
    double x2() const;
    double x3() const;

    // increment operators
    P& operator += (const P&);
    P& operator -= (const P&);
    P& operator *= (const double); // scalar multipication

    // cross product
    P& operator *= (const P&);

    // componentwise product
    P& operator &= (const P&);

    // orthogonalization: u %= v is the vector of the form u-k*v perp to v
    P& operator %= (const P&);

  private:
    double m_X1, m_X2, m_X3;
  }; // end of class P

  typedef P triple; // compatibility alias

  // standard basis in global namespace
  const P E_1(1,0,0);
  const P E_2(0,1,0);
  const P E_3(0,0,1);

  // vector space operations
  P operator- (const P& u);  // unary negation
  P operator+ (const P& u, const P& v);
  P operator- (const P& u, const P& v);
  // scalar multiplication
  P operator* (const double c, const P& v);

  P midpoint(const P& u, const P& v, const double t=0.5);

  // cross product
  P operator* (const P&, const P&);
  P J(const P&); // quarter turn about E_3-axis

  // dot product
  double operator | (const P&, const P&);
  double norm(const P&);

  // componentwise product (a,b,c)&(x,y,z)=(ax,by,cz)
  P operator& (const P&, const P&);
  // orthogonalization
  P operator% (const P&, const P&);

  // (in)equality
  bool operator == (const P&, const P&);
  bool operator != (const P&, const P&);

} // end of namespace

/*
 * functions.h  -- ePiX auxiliary functions
 *
 * This file is part of ePiX, a C++ library for creating high-quality 
 * figures in LaTeX
 *
 * Version 1.0.23
 * Last Change: January 29, 2007
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class P;

  // ePiX:: trig functions are sensitive to angle units
  double cos(double);
  double sin(double);
  double tan(double); 

  // non-mathematical behavior (i.e. vanishing:) at poles
  double sec(double);
  double csc(double);
  double cot(double);

  // and inverses
  double acos(double);
  double asin(double);
  double atan(double);

  // more sanely-named aliases
  double Cos(double t);
  double Sin(double t);
  double Tan(double t);

  double Sec(double t);
  double Csc(double t);
  double Cot(double t);

  double Acos(double t);
  double Asin(double t);
  double Atan(double t);

  // additional hyperbolic functions and inverses
  double sech(double);
  double csch(double);
  double coth(double);

  double asech(double);
  double acsch(double);
  double acoth(double);

  // identity and zero functions, coordinate projections
  template<class T>T id(T arg) { return arg; }
  template<class T>double zero(T arg) { return 0; }
  template<class T>double zero(T arg1, T arg2) { return 0; }

  template<class T1, class T2>T1 proj1(T1 arg1, T2 arg2) { return arg1; }
  template<class T1, class T2>T2 proj2(T1 arg1, T2 arg2) { return arg2; }

  // P constructors
  P xyz(double x, double y, double z=0);
  P cyl(double r, double t, double z);
  P sph(double r, double t, double phi);

  P cylindrical(P arg);
  P spherical(P arg);

  P polar(double r, double t);
  P cis(double t);

  // for log data plotting
  P log_log(double x, double y, double z=0);
  P log_lin(double x, double y, double z=0);
  P lin_log(double x, double y, double z=0);

  double clip_to_unit(double t);
  // utility functions with discontinuities removed
  double recip (double);
  double sinx (double);
  double sgn (double);

  // period-2 extension of absolute value on [-1,1]: \/\/\/
  double cb (double);

  int gcd (int, int);

  // Scattered calls compare (e.g.) int to float
  // template<class T>T min(T a, T b) { return a < b ? a : b; }
  // template<class T>T max(T a, T b) { return b < a ? a : b; }

  double min(const double, const double);
  double max(const double, const double);

  double snip_to(double var, const double arg1, const double arg2);

  double inf (double f(double), double, double);
  double sup (double f(double), double, double);


  /*
  // constant functions
  class Const {
  private:
    double value;

  public:
    Const(double val=0) { value = val; }
    double operator() (double t) { return value; }
  }; // end of class Const
  */

  // derivative and integral operators for functions of one variable
  template<class T> T deriv(T f(double), double t, double dt=EPIX_EPSILON)
  {
    return (1.0/dt)*(f(t+0.5*dt) - f(t-0.5*dt));
  }

  // derivative class
  class Deriv {
  private:
    double (*f)(double);
    double dt;

  public:
    Deriv(double func(double), const double eps=EPIX_EPSILON);

    P operator() (const P&) const; // for plotting

    // numerical values
    double eval(const double t) const;

    // one-sided derivatives
    double right(const double t) const;
    double left(const double t) const;
  }; // end of class Deriv


  // definite integral class
  class Integral {
  private:
    double (*f)(double);
    double x0; // lower limit

  public:
    Integral(double func(double), double a=0);

    P operator() (const P&) const;

    double eval(const double) const;
  }; // end of class I

  // compatibility aliases
  typedef Deriv D;
  typedef Integral I;

  double newton (double f(double), double g(double), double start);
  double newton (double f(double), double start);

} // end of namespace

/* 
 * frame.h -- Orthonormal basis
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 10, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * In geometry, a "frame" is a right-handed orthonormal basis, that is,
 * an ordered set of three mututally perpendicular unit vectors, oriented
 * according to the right-hand rule. A frame has nothing to do with
 * picture frames.
 *
 *
 * This file provides:
 *   - the frame class, and routines for rotating a frame about the axes
 *     determined by its elements. (Note that there are no methods for
 *     rotating about fixed coordinate axes, and that rotations in R^3 do
 *     not generally commute.
 */



namespace ePiX {

  class frame {
  public:
    // standard basis
    frame();

    // Gram-Schmidt
    frame(P arg1, P arg2, P arg3); // need args by value

    // frame elements
    P sea() const;
    P sky() const;
    P eye() const;

    // rotations about frame elements
    frame& rot1(const double angle);
    frame& rot2(const double angle);
    frame& rot3(const double angle);

  private:
    P m_e1; // sea
    P m_e2; // sky
    P m_e3; // eye
  }; // end of class frame

} // end of namespace

/* 
 * domain.h -- ePiX mesh and domain classes
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.1
 * Last Change: January 01, 2007
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */




namespace ePiX {

  class mesh {
  public:
    mesh(int n1, int n2, int n3=1);
    mesh(int n); // mesh(n, n, n);
    mesh();      // mesh(1, 1, 1);

    int n1() const { return n_1; }
    int n2() const { return n_2; }
    int n3() const { return n_3; }

  private:
    int n_1;
    int n_2;
    int n_3;
  }; // end of class mesh

  typedef mesh Net; // compatibility

  // A domain is a gridded coordinate box with specified corners
  // and coarse/fine meshes, used for plotting. The "coarse" step
  // sizes are step1, etc., and the "fine" step sizes are dx1, etc.
  // Steps may be negative.
  //
  // A domain may be "sliced" by setting one of its coordinates to
  // a specified constant.
  //
  // Data members are public for convenience; each member ensures
  // its own well-constructedness.
  //
  class domain {
  public:
    P corner1;
    P corner2;
    mesh coarse;
    mesh fine;

    domain(P arg1=P(1,1,1), P arg2=P(0,0,0), 
	   mesh c=mesh(1,1,1), mesh f=mesh(1,1,1));

    // 1-dim'l domain
    domain(double t_min, double t_max, int num_pts)
      : corner1(t_min), corner2(t_max), coarse(num_pts), fine(num_pts) { }

    int dim() const; // dimension

    // big and small steps
    double step1() const;
    double step2() const;
    double step3() const;

    double dx1() const;
    double dx2() const;
    double dx3() const;

    // resizing attempts to preserve real resolution
    domain resize1(double a1, double b1) const;
    domain resize2(double a2, double b2) const;
    domain resize3(double a3, double b3) const;

    // coordinate slices
    domain slice1(double a1) const;
    domain slice2(double a2) const;
    domain slice3(double a3) const;

    // "n=0": use number from coarse mesh
    std::list<domain> slices1(const unsigned int n=0) const;
    std::list<domain> slices2(const unsigned int n=0) const;
    std::list<domain> slices3(const unsigned int n=0) const;
  }; // end of class domain

  class domain_list {
  public:
    domain_list(std::list<domain>); // implicitly converts "slices" output
    domain_list& add(const domain&);
    domain_list& add(const domain_list&);

    std::list<domain> m_list; // public: we're a simple wrapper
  }; // end of class domain_list
} // end of namespace

/* 
 * pairs.h -- ePiX pair:: class and mathematical operators
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.7
 * Last Change: May 17, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class pair 
    {
    public:
      pair(const double arg1=0, const double arg2=0) : X1(arg1), X2(arg2) { }
      pair(const P& arg); // take first two coordinates

      double x1() const { return X1; }
      double x2() const { return X2; }

      // unary increment operators
      pair& operator += (const pair& arg);
      pair& operator -= (const pair& arg);
      pair& operator *= (const double c);

      // complex multiplication and division
      pair& operator *= (const pair& arg);
      pair& operator /= (const pair& arg);
      bool operator== (const pair& u) const;
      bool operator!= (const pair& u) const;

    private:
      double X1;
      double X2;

    }; // end of class pair
  
  pair operator- (pair u);
  pair operator+ (pair u, const pair& v);
  pair operator- (pair u, const pair& v);
  pair operator* (const double c, pair u);

  // complex arithmetic
  pair J(pair p);
  pair operator* (pair u, const pair& v);
  pair operator/ (pair u, const pair& v);

  // dot product
  double operator| (const pair& u, const pair& v);
  double norm (const pair& u);

  // componentwise product (a,b)&(x,y)=(ax,by)
  pair operator& (const pair& u, const pair& v);

} // end of namespace

/* 
 * camera.h -- the ePiX camera class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 10, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * ePiX uses a camera metaphor to view points in R^3. The camera consists
 * of:
 *   - a frame ({sea, sky, eye}, with the intuitive meanings:)
 *   - a point to look at (the "target")
 *   - the distance from the camera to the target
 *   - a projection mapping from R^3 to R^2 (the "lens")
 *
 * While specific behavior is determined by the lens, the idea is that the
 * viewer sits at given <distance> from the <target>, in the direction of
 * the <eye>. The vectors <sea> and <sky> point horizontally to the right
 * and vertically upward on the screen. A <distance> of zero is taken to
 * mean "infinitely far away"; for example, point projection (<shadow>)
 * becomes orthogonal projection along the <eye> axis. Again, a specific
 * lens can override this behavior, for example by ignoring the distance.
 *
 *
 * This file provides:
 *
 *   - the camera class and routines:
 *       void rotate_sea(double); // change camera orientation
 *       void rotate_sky(double); // about respective axes
 *       void rotate_eye(double);
 *
 *       void range(double d); // set distance from camera to origin
 *       void look_at(P arg); // set target (center of field of view)
 *       void lens(pair (*screen_projection(P))); // set projection mapping
 *
 *       pair shoot(P arg); // project a point to the screen ("take a photo")
 */



namespace ePiX {

  // screen projection mappings ("lenses")
  pair shadow(const P& arg);
  pair fisheye(const P& arg);
  pair bubble(const P& arg);

  class epix_camera {
  public:
    // defaults to orthog projection on (x,y) plane from pos z-axis
    epix_camera();

    P get_viewpt() const;
    P get_target() const;
    double get_range() const;

    // get frame vectors
    P sea() const;
    P sky() const;
    P eye() const;

    // adjust position, orientation, and target
    // pitch: rotate camera up/down
    epix_camera& rotate_sea(double angle);

    // yaw: rotate camera left/right
    epix_camera& rotate_sky(double angle);

    // roll: rotate camera about viewing axis
    epix_camera& rotate_eye(double angle);

    // fix target, move viewpt radially along eye()
    epix_camera& range(double d);
    // fix viewpt, move target radially along eye()
    epix_camera& focus(double d);

    // fix target, set viewpt arbitrarily
    epix_camera& at(const P& arg);
    epix_camera& at(const double, const double, const double);

    // fix viewpt, set target arbitrarily
    epix_camera& look_at(const P& arg);
    epix_camera& look_at(const double, const double, const double);

    // set projection mapping
    epix_camera& lens(pair proj(const P&));

    // project a point to the screen ("shoot a photo")
    pair operator() (const P&) const;

  private:
    P viewpt;
    P target;
    frame  orient;
    double distance;
    pair (*screen_projection)(const P&);

  }; // end of class epix_camera 

  extern epix_camera camera;

  // set global camera's location
  void viewpoint(const P&);
  void viewpoint(double a1, double a2, double a3);

} // end of namespace

/* 
 * cropping.h -- ePiX classes and functions for cropping paths and objects
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.18
 * Last Change: October 21, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class P;
  class path;

  // pre-declare friends for gcc-4.1
  // friends of crop_mask
  P seek_path_end(const P& in, const P& out);
  void bounding_box(const P&, const P&);
  void crop_box(const P&, const P&);
  void crop_box(); // (re)set crop box to bounding box

  void picture(const double, const double);
  void picture(const P&);
  void offset(const P& arg);
  void offset(const double hoff, const double voff);

  double h_size();
  double v_size();
  double h_offset();
  double v_offset();

  // friends of enclosure
  void clip_box(const P& arg1, const P& arg2);
  P seek_path_end(const P& in, const P& out);
  bool is_in_bounds(const P& arg);
  void clip_to(const P& arg);
  void clip_box(const P& arg); 

  double clip1_min();
  double clip2_min();
  double clip3_min();

  double clip1_max();
  double clip2_max();
  double clip3_max();
  // end of friends


  class crop_mask {
  public:
    static crop_mask Bounding_Box;
    static crop_mask Crop_Box;
    static crop_mask Picture;
    static pair Offset;

    crop_mask(const pair arg1=pair(1,1), const pair arg2=pair(0,0));

    double x_sz() const;
    double y_sz() const;

    pair bl() const;
    pair tr() const;
    //    pair size() const { return pair(x1_sz,  x2_sz); }

    // translate by pair
    crop_mask& operator+= (const pair arg);

    bool is_onscreen(const P& arg) const;

    // Given two points (one projecting to the crop_mask, one not),
    // return the point on the segment that projects to the border.
    P seek_crop(const P&, const P&) const;
    friend P seek_path_end(const P& in, const P& out);

    // functions in global scope for compatibility
    friend void bounding_box(const P&, const P&);

    friend void crop_box(const P&, const P&);
    friend void crop_box(); // (re)set crop box to bounding box

    friend void picture(const double, const double);
    friend void picture(const P&);

    friend void offset(const P& arg);
    friend void offset(const double hoff, const double voff);

    friend double h_size();
    friend double v_size();

    friend double h_offset();
    friend double v_offset();

  private:
    double x1_min, x1_max, x1_sz;
    double x2_min, x2_max, x2_sz;

  }; // end of class crop_mask


  class enclosure {
    friend class path;
  public:
    enclosure(const P& arg1, const P& arg2);
    //    bool is_cropped(const P arg);

    bool is_clipped1(const P&) const;
    bool is_clipped2(const P&) const;
    bool is_clipped3(const P&) const;
    bool is_inside(const P&) const;

    // Given two points, one in the enclosure, the other not, return the
    // point on the boundary.
    P seek_clip(const P& in, const P& out) const;

    // in global scope for compatibility
    friend void clip_box(const P& arg1, const P& arg2);

    friend P seek_path_end(const P& in, const P& out);

    friend bool is_in_bounds(const P&);
    friend void clip_to(const P&);
    friend void clip_box(const P&);

    friend double clip1_min();
    friend double clip2_min();
    friend double clip3_min();

    friend double clip1_max();
    friend double clip2_max();
    friend double clip3_max();

  private:
    double x1_min, x1_max;
    double x2_min, x2_max;
    double x3_min, x3_max;

    static enclosure Clip_Box;

  }; // end of class enclosure

  bool is_visible(const P&);

} // end of namespace

/* 
 *  path.h -- ePiX classes for polygons and paths
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 09, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class crop_mask;
  class enclosure;
  class sphere;

  class vertex {

  public:
    vertex(const double a1=0, const double a2=0, const double a3=0);
    vertex(const P& arg);

    P here() const;
    bool is_onscreen() const;
    bool is_in_world() const;

    void set_crop(const bool);
    void set_clip(const bool);

  private:
    P location;
    bool onscreen;
    bool in_world;
  }; //end of class vertex


  class path {
  public:

    path(const std::vector<vertex>& data, bool loop);
    path(unsigned int num_pts=0);

    // line constructors
    path(const P& tail, const P& head, const double expand=0);
    path(const P&, const P&, const double expand, unsigned int num_pts);

    // finite-data paths (ellipse, spline)
    path(const P& center, const P& axis1, const P& axis2, 
	 const double t_min, const double t_max,
	 unsigned int num_pts=EPIX_NUM_PTS);

    path(const P& p1, const P& p2, const P& p3, 
	 unsigned int num_pts=EPIX_NUM_PTS);

    path(const P& p1, const P& p2, const P& p3, const P& p4, 
	 unsigned int num_pts=EPIX_NUM_PTS);


    path(P f(double), double t_min, double t_max,
	 unsigned int num_pts=EPIX_NUM_PTS);
    path(double f(double), double t_min, double t_max, 
	 unsigned int num_pts=EPIX_NUM_PTS);

    // append a point
    path& pt(const double, const double, const double=0);
    path& pt(const P&);

    // concatenate path segments
    path& operator+= (const path& vertices);
    // concatenate, reversing second sequence
    path& operator-= (const path& vertices);

    bool is_closed() const { return closed; }
    bool is_filled() const { return filled; }

    // set attributes
    path& close(const bool arg=true);
    path& fill(const bool arg=true);
    path& set_fill(const bool arg=true); // deprecated

    void set_crop_all(const bool arg);
    void set_clip_all(const bool arg);

    // defined in output.cc
    void crop_to(const crop_mask& screen, const bool arg=true);
    void clip_to(const enclosure& world, const bool arg=true);
    //    void hide_by(bool hiding(const P));
    void draw();

    void draw(sphere, bool); // spherical plotting (front/back), in sphere.cc

  private:
    std::vector<vertex> vertices;

    bool closed;
    bool filled;
  }; // end of class path

  // polygon/polyline with variable number of vertices
  path polygon(unsigned int num_pts ...);
  path polyline(unsigned int num_pts ...);


  class path_pt {
  public:
    path_pt(const P& arg, bool is_start, bool is_end);
    path_pt(const vertex& arg, bool is_start, bool is_end);
    void unset();

    P here() const;
    bool is_start() const;
    bool is_end() const;

  private:
    P location;
    bool start; // start/end of path segment
    bool end;
  }; //end of class path_pt

} // end of namespace

/* 
 * segment.h -- ePiX::segment class
 * 
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 09, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The segment class (unordered pair of points),
 *   and operators:
 *    - endpt1(), endpt2() (no guaranteed order)
 *    - segment += P (translate by <P>)
 *    - draw() (ePiX line)
 *    - midpoint(); (also callable on pair of Ps)
 *    - segment*segment (pt of intersection of _lines_ determined by segments)
 */



namespace ePiX {

  class segment {
  public:
    segment() {};
    segment(const P&, const P&);

    // Segment ends are not distinguished geometrically,
    // so these functions should normally be used in tandem.
    P end1() const;
    P end2() const;

    // translate
    segment& operator += (const P&);

    P midpoint() const;

    void draw() const;

  private:
    P endpt1;
    P endpt2;
  }; // end of segment class


  // translate; "P + segment" form specifically omitted
  segment operator+ (const segment&, const P&);

  // intersect
  P operator* (const segment&, const segment&);

} // end of namespace
/* 
 * circle.h -- ePiX::circle class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 09, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The circle class (center, radius, unit normal) and operators. See
 *   function declarations for list of exceptions thrown
 *
 *    - circle() (defaults to origin, unit radius, in (x1,x2,0)-plane
 *    - circle(ctr, pt) (parallel to (x1,x2,0)-plane, center ctr, through pt)
 *    - circle(pt1, pt2, pt3) (arbitrary non-collinear points)
 *    - center(), radius(), perp()
 *    - circle += P (translate by <P>)
 *    - circle *= double (scale by <double>)
 *    - draw() (ePiX ellipse)
 *    - circle*segment, circle*circle (segment def'd by pts of intersection)
 */


namespace ePiX {

  class segment;

  class circle {
  public:
    // defaults to unit circle in (x1,x2,0) plane
    circle(const P& arg1=P(0,0,0), const double arg2=1, const P& arg3=E_3);

    circle(const P& ctr, const P& pt); // center, and point, normal = E_3
    circle(const P& pt1, const P& pt2, const P& pt3); // three points

    P center() const;
    double radius() const;
    P perp() const;

    // translation
    circle& operator+= (const P&);

    // scaling
    circle& operator*= (const double&);

    void draw() const;

  private:
    P ctr;
    double rad;
    P perp_to; // *unit* normal, even if rad = 0

  }; /* end of circle class */

  // intersection operators
  /*
   * Throws the following exceptions:
   *   NON_COPLANAR if segment, circle are not coplanar
   *   SEPARATED if coplanar but line defined by segment misses circle
   *   TANGENT if coplanar but line defined by segment tangent to circle
   */
  segment operator* (const segment&, const circle&);

  /*
   * Throws the following exceptions:
   *   TANGENT if circles are tangent
   *   COINCIDENT if circles are identical
   *   NOT_COPLANAR if circles not coplanar
   *   SEPARATED if disjoint, neither contained in the other
   *   CONCENTRIC if disjoint, one contained in other (not nec same ctr)
   */
  segment operator* (const circle&, const circle&);
  segment operator* (const circle&, const segment&);

  // affine operations: translation by a triple...
  circle operator+ (const circle&, const P&);

  // and scaling by a double
  circle operator* (const double&, const circle&);
  circle operator* (const circle&, const double&);

} // end of namespace

/* 
 * sphere.h -- ePiX::sphere class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 09, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *   This file provides:
 *
 *   The sphere class (center, radius),
 *   and operators:
 *    - sphere += P (translate by <P>)
 *    - sphere *= double (scale about center)
 *    - draw() (ePiX line)
 *    - center();
 *    - radius();
 *    - sphere*sphere (circle of intersection)
 *
 *   Hiding test, latitude and longitudes on a sphere.
 */


namespace ePiX {

  class circle;

  class sphere {
  public:
    sphere(const P& p1=P(0,0,0), double r=1);
    sphere(const P& p1, const P& p2);

    P center() const;
    double radius() const;

    // translation
    sphere& operator+= (const P&);

    // scaling
    sphere& operator*= (const double&);

    void draw() const;

  private:
    P ctr;
    double rad;
  }; // end of sphere class


  // translation
  sphere operator+ (const sphere&, const P&);

  // scaling about center
  sphere operator* (const double&, const sphere&);

  // intersection
  circle operator* (const sphere&, const sphere&);


  // named constructor: specify pair of antipodal points
  sphere poles(const P& p1, const P& p2);


  // utility function for hidden line removal (hidden<0 reverses value)
  bool visible_on_sphere(const P& arg, bool front = true, 
			 const sphere& S=sphere());

  // {lat,long}itude lines with hiding
  void latitude(double lat, double long_min, double long_max,
		sphere S=sphere(), frame coords=frame());

  void longitude(double lngtd, double lat_min, double lat_max,
		 sphere S=sphere(), frame coords=frame());

  void back_latitude(double lat, double long_min, double long_max,
		     sphere S=sphere(), frame coords=frame());

  void back_longitude(double lngtd, double lat_min, double lat_max,
		      sphere S=sphere(), frame coords=frame());

} // end of namespace

/* 
 *  plane.h -- ePiX::plane class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 09, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *  This file provides:
 *
 *  The plane class (point, normal),
 *  and operators:
 *   - height function and friends
 *   - equality, parallelity test
 *   - plane*segment (point of intersection)
 *   - plane*plane (draw line of intersection)
 *   - plane*sphere (circle of intersection)
 *
 */



namespace ePiX {

  class circle;
  class segment;
  class sphere;

  class plane {
  public:
    plane(const P& point=P(0,0,0), const P& normal=E_3);
    plane(const P&, const P&, const P&);

    P normal() const;
    plane& reverse();
    plane& operator+= (const P&);

    // normal component of arg
    double height(const P&) const;
    bool contains(const P&) const;

    // pts separated by us, or we contain at least one point
    bool separates(const P&, const P&) const;
    bool parallel_to (const plane&) const;
    bool operator== (const plane&) const;

    // return circle of intersection
    circle operator* (const sphere&) const;
    // draw Line
    void  operator* (const plane&) const;

    // draw lines of intersection of *this with clip box faces
    void draw() const;

  private:
    P m_pt;
    P m_perp; // UNIT normal
  }; // end of plane class

  circle operator* (const sphere&, const plane&);

  P operator* (const plane&, const segment&);
  P operator* (const segment&, const plane&);

} // end of namespace

/* 
 *  polyhedron.h -- ePiX:: convex polyhedron class
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.4
 * Last Change: March 13, 2005
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 *  This file provides:
 *
 *  The one_skel (topological 1-skeleton) class (<vector> of segments) 
 *  and operators:
 *    transform() // apply mapping to each vertex
 *    section()   // draw planar cross section
 */




namespace ePiX {

  class one_skel {
  private:
    // std::vector<P> vertices;
    std::vector<segment> edges;

  public:
    one_skel();
    one_skel(unsigned int num_edges ...); // list of edges

    one_skel(const one_skel& old);

    //    void add_vertex(const P arg); // add vertex, form convex hull
    void add_edge(const segment arg) { edges.push_back(arg); }

    void transform(P Phi(P)); // apply Phi to each vertex
    // void draw();
    void section(const plane& knife);
  }; // end of class one_skel

} /* end of namespace */

/*
 * output.h -- ePiX file-formatting output macros
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 10, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class P;
  class pair;

  void unitlength(const char *units);

  // Basic Sectioning Functions

  // Begin/end picture
  void begin();
  void end();

  // print truncated pair/triple coordinates  
  void raw_print(const pair&);

  void print(const P&);
  void print(const pair&);
  void print(const P& location, const P& offset);
  void print_vector(const P&);

  void print_alignment(epix_label_posn POSN);
  void print_marker(epix_mark_type MARK);

  // Functions that write small pieces of a LaTeX \picture environment
  void lbrace(); // {
  void rbrace(); // }
  void dollar(); // $
  void newl();   // \n

  // common \LaTeX commands

  // print arg in braces
  void epix_grouping(const double);
  void epix_grouping(const std::string&);

  // same, in math mode
  void epix_math_grouping(const double);
  void epix_math_grouping(const std::string&);

  // label txt, with font information
  void epix_label_text(const std::string& arg);

  void epix_neg(); // phantom{-}
  void epix_put();
  void epix_multiput();
  void epix_makebox();
  void epix_whiten();

  void start_path();
  void end_path();

  // string-valued functions for path::draw()
  std::string start_path_string();
  std::string end_path_string();

  void psset(const std::string&);
  void fill_color(const std::string& color="white");

  void epix_rotatebox();
  void epix_colorbox();

  void epix_rule(const double x, const double y);

  void epix_circle(const double r_pt); // radius in pt
  void epix_disk(const double r_pt);

  void epix_newline(); // \n%%
  void epix_newline(const std::string&);

  void end_stanza();
  void end();
  void line_break(int, int);

  // Misc Styles and output formatting
  // Output styles
  void pen(const std::string&);
  void pen(double);

  void bold();
  void plain();
} // end of namespace

/*
 * colors.h -- ePiX color macros
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.15
 * Last Change: October 10, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

namespace ePiX {

  // ePiX provides rgb/cmyk colors via pstcol. The following are equivalent:
  //    red();
  //    rgb(1,0,0);
  //
  // cmyk(0,1,1,0); is visually equivalent but not the same.
  //
  // Densities outside the unit interval [0,1] are truncated, e.g., 
  //    rgb(2, -1, 0.3) = rgb(1, 0, 0.3).

  // red-green-blue
  void rgb(double r, double g, double b); // must pass args by value
  // cyan-magenta-yellow-black
  void cmyk(double c, double m, double y, double k);

  // primary colors
  void red(double d=1);
  void green(double d=1);
  void blue(double d=1);
  void white(double d=1);
  void black(double d=1);

  void cyan(double d=1);
  void magenta(double d=1);
  void yellow(double d=1);

} // end of namespace

/* 
 * objects.h -- ePiX axes, grids, markers, and labels
 *
 * This file is part of ePiX, a C++ library for creating high-quality 
 * figures in LaTeX 
 *
 * Version 1.0.23
 * Last Change: January 04, 2007
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * label -- put string constant <label_text> at Cartesian position <base>
 * translated by (offset.x1, offset.x2) true points (i.e., 3rd component
 * of <offset> is discarded).
 *
 * Accepts an optional LaTeX-style positioning argument.
 * If no offset is specified, the label is centered at the given Cartesian
 * location.
 * masklabel requires the "color" package, and places the text in 
 * a white box that masks whatever is underneath and earlier in the file.
 */



namespace ePiX {

  void label(const P& base, const P& offset, std::string label_text);
  void label(const P& base, std::string label_text);
  void label(const P& base, const P& offset, std::string label_text,
	     epix_label_posn);

  void masklabel(const P& base, const P& offset, std::string label_text);
  void masklabel(const P& base, std::string label_text);
  void masklabel(const P& base, const P& offset, std::string label_text,
		 epix_label_posn);

  // Empty and filled LaTeX circles of diameter get_dotsize() true pt
  void circ(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none); // filled white circ

  void ring(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none); // unfilled circ 

  void spot(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none);

  void dot(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none);

  void ddot(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none);

  void box(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none);

  void bbox(const P& posn, const P& offset=P(0,0), std::string label_text="",
	    epix_label_posn align=none);

  void arrow(const P& tail, const P& head, const P& offset,
	     std::string label_text,
	     epix_label_posn align, double scale=1);

  void marker (const P&, epix_mark_type);

  // Axes and coordinate grids.

  // Coordinate axes, specified by initial and final points, number of
  // tick marks, and (optionally) number of points (for cameras that do
  // not map lines to lines). Generally num_pts should be a multiple of n.
  // h/v_axis are identical except for style of tick marks.

  void h_axis_tick(const P& location);
  void v_axis_tick(const P& location);

  // n+1 = #ticks, num_pts = #segments used to draw
  void h_axis(const P& tail, const P& head, int n, int num_pts=1);
  void v_axis(const P& tail, const P& head, int n, int num_pts=1);

  // Endpoints (x_min,0), x_max,0), etc.
  void h_axis(int n=x_size);
  void v_axis(int n=y_size);

  // n+1 = #ticks, num_pts = #segments used to draw
  void h_axis(int n, int num_pts);
  void v_axis(int n, int num_pts);


  // h_axis_labels: Draws n+1 equally-spaced axis labels between 
  //   <tail> and <head>. Automatically generates label values from
  //   Cartesian coordinates, and offsets labels in true pt.

  void h_axis_labels(const P& tail, const P& head, int n, const P& offset);
  void v_axis_labels(const P& tail, const P& head, int n, const P& offset);

  void h_axis_masklabels(const P& tail, const P& head, int n, const P& offset);
  void v_axis_masklabels(const P& tail, const P& head, int n, const P& offset);

  // Axis labels with LaTeX-style alignment option

  void h_axis_labels(const P& tail, const P& head, int n, 
		     const P& offset, epix_label_posn POSN);
  void v_axis_labels(const P& tail, const P& head, int n, 
		     const P& offset, epix_label_posn POSN);

  void h_axis_masklabels(const P& tail, const P& head, int n, 
			 const P& offset, epix_label_posn POSN);
  void v_axis_masklabels(const P& tail, const P& head, int n, 
			 const P& offset, epix_label_posn POSN);

  // Axis labels with default endpoints
  void h_axis_labels(int n, const P& offset);
  void h_axis_masklabels(int n, const P& offset);

  void h_axis_labels(int n, const P& offset, epix_label_posn POSN);
  void h_axis_masklabels(int n, const P& offset, epix_label_posn POSN);

  void v_axis_labels(int n, const P& offset);
  void v_axis_masklabels(int n, const P& offset);

  void v_axis_labels(int n, const P& offset, epix_label_posn POSN);
  void v_axis_masklabels(int n, const P& offset, epix_label_posn POSN);

  // labels written $10^i$ at coord i
  void h_axis_log_labels(const P& tail, const P& head,
			 const P& offset, epix_label_posn POSN);

  void v_axis_log_labels(const P& tail, const P& head,
			 const P& offset, epix_label_posn POSN);

  void h_log_axis(const P& tail, const P& head,
		  unsigned int segs, unsigned int base=10);

  void v_log_axis(const P& tail, const P& head,
		  unsigned int segs, unsigned int base=10);

} // end of namespace

/*
 * curves.h -- Ellipses, arcs, splines, coordinate grids
 *
 * This file is part of ePiX, a C++ library for creating high-quality
 * figures in LaTeX
 *
 * Version 1.0.23
 * Last Change: January 04, 2007
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class P;
  class mesh;

  extern double x_min, y_min, x_max, y_max;

  // lines can be "stretched" by double parameter
  void line(const P& tail, const P& head, const double expand=0); 
  void line(const P& tail, const P& head, const double expand,
	    int num_pts);

  // "Visible" portion of the line through p1, p2
  void Line(const P& tail, const P& head);

  // point-slope form
  void Line(const P&, const double);

  void triangle(const P&, const P&, const P&);
  void quad(const P&, const P&, const P&, const P&);

  void draw_rect(const P&, const P&); // opposite corners
  void rect(const P&, const P&, bool solid=epix::fill_paths);
  // filled rectangle; deprecated
  void swatch(const P&, const P&);

  // arrows
  void arrow(const P& tail, const P& head, double scale=1);
  void dart(const P& tail, const P& head);

  // double-tipped
  void aarrow(const P& tail, const P& head, double scale=1);

  // Algebraic curves (elliptical and circular arcs, splines)
  void ellipse(const P& center, const P& axis1, const P& axis2,  
	       const double t_min=0, 
	       const double t_max=epix::full_turn(),
	       int num_pts=EPIX_NUM_PTS);

  // for backward compatibility
  void ellipse_arc(const P& center, const P& axis1, const P& axis2,
		   const double t_min, const double t_max);

  // old style "center and polyradius"
  void ellipse (const P& center, const P& radius);

  // Standard half-ellipse functions
  void ellipse_left (const P&, const P&);
  void ellipse_right (const P&, const P&);
  void ellipse_top (const P&, const P&);
  void ellipse_bottom (const P&, const P&);

  // Circular arcs parallel to (x,y)-plane
  void arc(const P& center, const double r, 
	   const double start,  const double finish);

  void arc_arrow (const P& center, const double r, 
		  const double start, const double finish, 
		  const double scale=1);


  // Quadratic and cubic splines/spline arrows
  void spline(const P& p1, const P& p2, const P& p3, int num_pts=EPIX_NUM_PTS);
  void arrow(const P& p1, const P& p2, const P& p3, double scale=1);

  void spline (const P& p1, const P& p2, const P& p3, const P& p4,
	       int num_pts=EPIX_NUM_PTS);
  void arrow(const P&, const P&, const P&, const P&, double scale=1);


  // A "mesh" is an ordered pair of positive integers, and is used to
  // specify the "fineness" of a mesh. Grids, like parametric surface
  // meshes, have a "coarse mesh" -- the numbers of grid intervals in
  // each direction, and a "fine mesh" -- the numbers of points used
  // to render the grid lines. Since an ePiX camera does not always
  // map lines in object space to lines on the screen, grid lines cannot
  // generally be drawn using only two points.
  // A grid may look strange unless each component of fine is a multiple
  // of the corresponding entry of coarse, of course. :)

  // Cartesian grid of specified size, mesh, and resolution
  void grid(const P& arg1, const P& arg2, mesh coarse, mesh fine);

  // coarse = fine = (n1,n2)
  void grid(const P& arg1, const P& arg2, int n1=1, int n2=1);
  void grid(int n1=1, int n2=1);

  // polar grid of specified radius, mesh (rings and sectors), and resolution
  void draw_polar_grid(double r, mesh coarse, mesh fine);
  void polar_grid(double r, mesh coarse, mesh fine);

  // polar grid with n1 rings and n2 sectors
  void polar_grid(double r, int n1, int n2);


  // (semi-)logarithmic grids specified by corners and numbers of orders of
  // magnitude or grid divisions in each direction. Optional arguments
  // specify the log base (10 by default). If corners are omitted, the grid
  // fills the bounding box.
  void log_grid(const P& arg1, const P& arg2,
		unsigned int segs1, unsigned int segs2,
		unsigned int base1=10, unsigned int base2=10);

  void log1_grid(const P& arg1, const P& arg2,
		 unsigned int segs1, unsigned int segs2,
		 unsigned int base1=10);

  void log2_grid(const P& arg1, const P& arg2,
		 unsigned int segs1, unsigned int segs2,
		 unsigned int base2=10);

  void log_grid(unsigned int segs1, unsigned int segs2,
		unsigned int base1=10, unsigned int base2=10);

  void log1_grid(unsigned int segs1, unsigned int segs2,
		 unsigned int base1=10);

  void log2_grid(unsigned int segs1, unsigned int segs2,
		 unsigned int base2=10);

  // fractal generation
  //
  // The basic "level-1" recursion unit is a piecewise-linear path whose 
  // segments are parallel to spokes on a wheel, labelled modulo <spokes>.
  // Recursively up to <depth>, each segment is replaced by a copy of the
  // recursion unit.
  //
  // Sample data for _/\_ standard Koch snowflake:
  // const int pre_seed[] = {6, 4, 0, 1, -1, 0};
  // pre_seed[0] = spokes, pre_seed[1] = seed_length;

  void fractal (const P& p, const P& q, const int depth, const int *pre_seed);

} // end of namespace

/*
 * plots.h -- parametrized plotting algorithms
 *
 * Andrew D. Hwang   <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 *
 * Version 1.1
 * Last Change: January 01, 2007
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

namespace ePiX {

  class domain;
  class domain_list;
  class Deriv;
  class Integral;
  class P;

  //// paths ////
  // f:R -> R^3
  void plot (P f(double), double t_min, double t_max, int num_pts);

  // f:R -> R x R x R
  void plot (double f1(double), double f2(double), double f3(double),
	     double t_min, double t_max, int num_pts);

  // f:R -> R
  void plot (double f(double), double t_min, double t_max, int num_pts);
  void plot (const Deriv&, double t_min, double t_max, int num_pts);
  void plot (const Integral&, double t_min, double t_max, int num_pts);

  // f: R -> R x R
  void plot (double f1(double), double f2(double), 
	     double t_min, double t_max, int num_pts);

  void polarplot (double f(double), double t_min, double t_max, int num_pts);
  
  //// surfaces ////
  // f:R^2 -> R x R x R
  void plot(double f1(double u1, double u2),
	    double f2(double u1, double u2),
	    double f3(double u1, double u2), 
	    const P& min, const P& max, const mesh& coarse, const mesh& fine);

  void plot(double f1(double u1, double u2),
	    double f2(double u1, double u2),
	    double f3(double u1, double u2), 
	    const domain& R);

  // f:R^2 -> R
  void plot(double f(double u1, double u2),
	    const P& p1, const P& p2, const mesh& coarse, const mesh& fine);

  // f:R^2 -> R^3
  void plot(P f(double, double), const domain&);
  void plot(P f(double, double), const domain_list&);

  // f:R^3 -> R^3
  void plot(P f(double, double, double), const domain&);
  void plot(P f(double, double, double), const domain_list&);


  // Derivatives and integrals
  void plot_deriv(double f(double), double a, double b, int num_pts);

  void plot_int(double f(double), double a, double b, int num_pts);

  void plot_int(double f(double), double x0, double a, double b, int num_pts);

  void tan_line(double f1(double), double f2(double), double t0);
  void tan_line(double f(double), double t0);
  void tan_line(P f(double), double t0);

  void envelope(double f1(double), double f2(double), double, double, int);
  void envelope(double f(double), double, double, int);
  void envelope(P f(double), double, double, int);

  void tan_field(double f1(double), double f2(double), double, double, double);
  void tan_field(P f(double), double, double, double);


  // Slope, dart, and vector fields
  // For slope and dart fields, the optional scale argument affects the
  // (constant) drawn length of field elements. For vector fields, elements
  // are drawn at true length, and "scale" affects the arrowhead size.

  // planar fields
  void  slope_field(P F(double, double), const P& p, const P& q, int, int,
		    const double scale=1.0);
  void   dart_field(P F(double, double), const P& p, const P& q, int, int,
		    const double scale=1.0);
  void vector_field(P F(double, double), const P& p, const P& q, int, int,
		    const double scale=1.0);

  void  slope_field(P F(double, double), const domain& R,
		    const double scale=1.0);
  void   dart_field(P F(double, double), const domain& R,
		    const double scale=1.0);
  void vector_field(P F(double, double), const domain& R,
		    const double scale=1.0);

  // spatial fields
  void  slope_field(P F(double, double, double), const P&, const P&, int, int,
		    const double scale=1.0);
  void   dart_field(P F(double, double, double), const P&, const P&, int, int,
		    const double scale=1.0);
  void vector_field(P F(double, double, double), const P&, const P&, int, int,
		    const double scale=1.0);

  // spatial fields over a (3-D) domain
  void  slope_field(P F(double, double, double), const domain&,
		    const double scale=1.0);
  void   dart_field(P F(double, double, double), const domain&,
		    const double scale=1.0);
  void vector_field(P F(double, double, double), const domain&,
		    const double scale=1.0);


  // Solutions of ODE systems

  // start at time 0
  void ode_plot (P F(double, double),
		 const P& start, double t_max, int num_pts);

  void ode_plot (P F(double, double, double),
		 const P& start, double t_max, int num_pts);

  // arbitrary start time
  void ode_plot (P F(double, double),
		 const P& start, double t_min, double t_max, int num_pts);

  void ode_plot (P F(double, double, double),
		 const P& start, double t_min, double t_max, int num_pts);

  // flow x0 under field for specified time
  P flow (P F(double, double), const P& start, double t_max, int num_pts = 0);
  P flow (P F(double, double, double),
	  const P& start, double t_max, int num_pts = 0);

  void riemann_sum(double f(double), const double a, const double b,
		   unsigned int n, epix_integral_type TYPE);


  // Jay Belanger's shaded plot functions -- December 1, 2002
  void shadeplot(double f1(double), double f2(double), double, double, int);
  void shadeplot(double f(double), double t_min, double t_max, int num_pts);

  void blackplot(double f1(double), double f2(double), 
		 double t_min, double t_max, int num_pts);

  void blackplot(double f(double), double t_min, double t_max, int num_pts);

  void whiteplot(double f1(double), double f2(double), 
		 double t_min, double t_max, int num_pts);

  void whiteplot(double f(double), double t_min, double t_max, int num_pts);


  // wiremesh surface plotting
  void plot(P Phi(double, double),
	    const P&, const P&, const mesh& N, const mesh& num_pts);

  void clipplot(P Phi(double, double),
	    const P&, const P&, const mesh& N, const mesh& num_pts);

  void cropplot(P Phi(double, double),
		const P&, const P&, const mesh& N, const mesh& num_pts);

} // end of namespace

/*
 * surface.h -- Shaded surface plotting
 *
 * This file is part of ePiX, a preprocessor for creating high-quality
 * line figures in LaTeX
 *
 * Version 1.0.24
 * Last Change: February 27, 2007
 */

/*
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX 
{
  class domain;
  class domain_list;
  class facet;

  // manipulable collection of surface-like objects
  class scenery {
  public:
    scenery();
    scenery(P F(double, double), const domain& R);
    scenery(P F(double, double, double), const domain& R);

    scenery(const scenery&);
    scenery& operator= (const scenery&);
    ~scenery();

    scenery& add(P F(double, double), const domain& R);
    scenery& add(P F(double, double, double), const domain& R);

    scenery& add(P F(double, double), const domain_list&);
    scenery& add(P F(double, double, double), const domain_list&);

    // remove backward-pointing faces? 0=no, -1=front, 1=back
    scenery& cull(int);
    void draw(int cull=0); // not const: must sort facets

  private:
    int m_cull;
    std::list<facet*> m_data;
  }; // end of class scenery

  // except as noted, dim(R) must be 2
  // cosine-shaded surface with fake z-buffered hidden object removal
  void surface(P F(double, double), const domain& R, int cull=0);

  // for slices of maps R^3 -> R^3
  void surface(P F(double, double, double), const domain& R, int cull=0);

  // plot multiple slices
  void surface(P F(double, double, double), const domain_list& R, int cull=0);

  // Surface from revolving the curve (f(t),g(t)) about the x-axis...
  void surface_rev(double f(double), double g(double),
		   double t_min, double t_max, 
		   int latitudes, int longitudes=24, int cull=0);

  void surface_rev(double f(double),
		   double t_min, double t_max, 
		   int latitudes, int longitudes=24, int cull=0);

  // or about the specified axis, with specified angle range
  void surface_rev(double f(double), double g(double), 
		   const domain& R, int cull=0, const frame& coords=frame());
} // end of namespace ePiX
/* 
 *  dataplot.h -- ePiX::P data plotting and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.23
 * Last Change: February 11, 2007
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


namespace ePiX {

  class d_bin;

  /*
  // wrapper for bool-valued function of P
  class selection {
  public:
    selection();
    selection(bool F(P));

    bool operator() (const P&) const;

  private:
    bool (*m_selector)(P);

  }; // end of class selection

  selection& all();
  */

  // safer, more featureful replacement for "FILEDATA"
  class data_file {
  public:
    explicit data_file(unsigned int n=2);
    explicit data_file(const char*, const char* delim = " ",
    	const char* commt = "%"); // get size from first line of file

    // file made from components
    data_file(double f(double),
	      double t_min, double t_max, unsigned int num_pts);

    data_file(double f1(double), double f2(double), 
	      double t_min, double t_max, unsigned int num_pts);

    data_file(double f1(double), double f2(double), double f3(double),
	      double t_min, double t_max, unsigned int num_pts);

    // input
    data_file& read(const char*);

    // transform column(s)
    data_file& transform(double f(double), unsigned int col=0);

    // apply f to selected columns; components of image go back to columns
    data_file& transform(P f(double, double),
			 unsigned int col1=1, unsigned int col2=2);

    data_file& transform(P f(double, double, double),
			 unsigned int col1, unsigned int col2);

    data_file& transform(P f(double, double, double),
			 unsigned int col1,
			 unsigned int col2,
			 unsigned int col3);

    // basic statistical operations on columns
    double dot(unsigned int col1, unsigned int col2) const; // dot product
    double avg(unsigned int col1) const;  // mean
    double var(unsigned int col1) const;  // |x|^2 - n*avg^2
    double covar(unsigned int col1, unsigned int col2) const;
    void   regression(unsigned int col1, unsigned int col2) const;


    // output functions
    // extract column
    std::vector<double> column(unsigned int) const;
    // apply f to values
    std::vector<double> column(double f(double), unsigned int) const;

    // set C++ output precision for write
    void precision(unsigned int n=6) const;

	// Setters and getters for delimiter and comment character
	void delimiter(const char* delim) { m_delim = delim; }
	std::string delimiter() { return m_delim; }
	void comment(const char* commt) { m_commt = commt; }
	std::string comment() { return m_commt; }

    // write raw data to file
    void write(const char* filename) const;

    // write selected columns formatted by string-valued function
    void write(const char* filename, std::string pt_out(double, double),
	       unsigned int col1=1, unsigned int col2=2) const;

    // LaTeX tabular environment
    void tabular(const char* filename,
		 const std::string& alignment,
		 const std::string& legend="") const;


    /*
    // (un)set cropping criterion for plotting
    void select(bool Fsel(P));
    void select(const selection&); // e.g. select(all());
    */

    // scatter plots; f applied to selected columns
    void plot(epix_mark_type TYPE,
	      unsigned int col1=1, unsigned int col2=2, unsigned int col3=0,
	      P f(double, double, double) = xyz) const;

    void plot(epix_mark_type TYPE, P f(double, double, double),
	      unsigned int col1=1, unsigned int col2=2,
	      unsigned int col3=0) const;

  private:
	std::vector<double> tokenise(std::string line);
	unsigned int entries(const char* filename);

    mutable unsigned int m_precision;
    // selection m_select;

    std::vector<std::vector<double> > m_data;
    std::string m_delim;		// Field delimiter
    std::string m_commt;		// Comment character
  }; // end of class data_file


  class data_bins {
  public:
    data_bins(const double lo, const double hi, unsigned int n=1);

    data_bins(const data_bins&);
    data_bins& operator= (const data_bins&);
    ~data_bins();

    data_bins&  cut(const double); // add a cut
    data_bins& read(const std::vector<double>&);

    unsigned int pop() const; // current population

    void histogram(const double scale=1) const; // draw rectangles
    void plot(const double scale=1) const;  // smooth interpolation

  private:
    double m_lo_val;
    double m_hi_val;

    unsigned int m_lo_ct; // low count
    unsigned int m_hi_ct; // high count
    unsigned int m_pop; // current population

    bool m_cuts_locked; // true once we read data

    std::list<double> m_cuts;
    std::vector<d_bin*> m_bins;

    // bookkeeping
    void initialize();         // convert cuts to bins and lock
    void insert(const double); // add data point
  }; // end of class data_bins


  //// Global functions that use file_data, data_bins ////
  void plot(const char* filename, epix_mark_type TYPE,
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0,
	    P f(double, double, double) = xyz);

  void plot(const char* filename, epix_mark_type TYPE,
	    P f(double, double, double),
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0);

  // pass 3rd arg by value
  void histogram(const char* filename, unsigned int col, data_bins);


  ///////////////////////////////////////////////////
  //// OBSOLETE FUNCTION CALLS -- may be removed //// 
  ///////////////////////////////////////////////////
  typedef std::vector<std::vector<double> > FILEDATA;

  // read columns of file into FILEDATA
  void read(const char*, FILEDATA&);

  // plot specified columns
  // Primarily for raw data
  void plot(const FILEDATA& data_columns, epix_mark_type TYPE,
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0,
	    P f(double, double, double) = xyz);

  void plot(const char* filename, epix_mark_type TYPE, int columns, 
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0,
	    P f(double, double, double) = xyz);


  // Primarily for log plots
  void plot(const char* filename, epix_mark_type TYPE, int columns, 
	    P f(double, double, double),
	    unsigned int col1=1, unsigned int col2=2, unsigned int col3=0);


  // deprecated placeholder
  void data_plot(const char* filename, epix_mark_type TYPE);

  // functions for data analysis
  double avg(FILEDATA data, unsigned int col1);
  double var(FILEDATA data, unsigned int col1);
  double covar(FILEDATA data, unsigned int col1, unsigned int col2);

  void regression(FILEDATA data, unsigned int col1=1, unsigned int col2=2);

  // Histogram plot of designated column, in the specified interval with
  // equal-length data subdivisions. By default the total area is unity;
  // v_scale adjusts height.
  void histogram(const FILEDATA& data_columns, unsigned int column, 
		 int subdivs=(int)x_size, 
		 double low=x_min, double high=x_max, double v_scale=1);

  void histogram(const char* filename, int columns, unsigned int column, 
		 int subdivs=(int)x_size, 
		 double low=x_min, double high=x_max, double v_scale=1);

} // end of namespace

/*
 * geometry.h 
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 1.0.20
 * Last Change: October 29, 2006
 */

/* 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


namespace ePiX {
  class P;
  class sphere;

  // All sphere line functions treat point arguments as unscaled
  // displacements from the center of the sphere, not as absolute
  // spatial locations.  If the center or radius of the sphere
  // changes, the line automatically adjusts as expected.

  // front portion of spherical segment joining p1 to p2
  void front_arc(const P& p1, const P& p2, const sphere& S = sphere());
  void  back_arc(const P& p1, const P& p2, const sphere& S = sphere());

  // half-line joining p1 to -p1 through p2
  void front_arc2(const P&, const P&, const sphere& S=sphere());
  void  back_arc2(const P&, const P&, const sphere& S=sphere());

  // great circle through p1 and p2
  void front_line(const P&, const P&, const sphere& S=sphere());
  void  back_line(const P&, const P&, const sphere& S=sphere());

  void front_triangle(const P&, const P&, const P&, const sphere& S=sphere());
  void  back_triangle(const P&, const P&, const P&, const sphere& S=sphere());

  // stereographic projection; path first projected vertically
  void frontplot_N(double f1(double), double f2(double),
		   double t_min, double t_max, int num_pts, 
		   const sphere& S = sphere());

  void backplot_N(double f1(double), double f2(double),
		  double t_min, double t_max, int num_pts,
		  const sphere& S = sphere());

  void frontplot_S(double f1(double), double f2(double),
		   double t_min, double t_max, int num_pts, 
		   const sphere& S = sphere());

  void backplot_S(double f1(double), double f2(double),
		  double t_min, double t_max, int num_pts, 
		  const sphere& S = sphere());

  // Radial projection from center
  void frontplot_R(P phi(double), double t_min, double t_max, 
		   int num_pts, const sphere& S = sphere());

  void backplot_R(P phi(double), double t_min, double t_max, 
		  int num_pts, const sphere& S = sphere());

  // spherical polyhedra
  void front_tetra(const sphere& S=sphere(), const frame& coords=frame());
  void  back_tetra(const sphere& S=sphere(), const frame& coords=frame());

  void front_cube(const sphere& S=sphere(), const frame& coords=frame());
  void  back_cube(const sphere& S=sphere(), const frame& coords=frame());

  void front_octa(const sphere& S=sphere(), const frame& coords=frame());
  void  back_octa(const sphere& S=sphere(), const frame& coords=frame());

  void front_dodeca(const sphere& S=sphere(), const frame& coords=frame());
  void  back_dodeca(const sphere& S=sphere(), const frame& coords=frame());

  void front_icosa(const sphere& S=sphere(), const frame& coords=frame());
  void  back_icosa(const sphere& S=sphere(), const frame& coords=frame());

  // Hyperbolic lines
  void hyperbolic_line (const P&, const P&);
  void disk_line (const P&, const P&);

} // end of namespace


#endif /* EPIX_H */
